# ANSI style codes
ANSI_STYLES = {
    # Reset all styles
    'reset': 0,

    # Text styles
    'bold'          : 1,
    'dim'           : 2,
    'italic'        : 3,
    'underline'     : 4,
    'blinking'      : 5,
    'reverse'       : 7,
    'hidden'        : 8,
    'strikethrough' : 9,

    # reset text styles
    'rbold'         : 22,
    'rdim'          : 22,
    'ritalic'       : 23,
    'runderline'    : 24,
    'rblinking'     : 25,
    'rreverse'      : 27,
    'rhidden'       : 28,
    'rstrikethrough': 29,

    # Foreground colors
    'black'         : 30,
    'red'           : 31,
    'green'         : 32,
    'yellow'        : 33,
    'blue'          : 34,
    'magenta'       : 35,
    'cyan'          : 36,
    'white'         : 37,

    # Background colors
    'bblack'        : 40,
    'bred'          : 41,
    'bgreen'        : 42,
    'byellow'       : 43,
    'bblue'         : 44,
    'bmagenta'      : 45,
    'bcyan'         : 46,
    'bwhite'        : 47,

    # Light foreground colors
    'lblack'        : 90,
    'lred'          : 91,
    'lgreen'        : 92,
    'lyellow'       : 93,
    'lblue'         : 94,
    'lmagenta'      : 95,
    'lcyan'         : 96,

    # Light background colors
    'blblack'       : 100,
    'blred'         : 101,
    'blgreen'       : 102,
    'blyellow'      : 103,
    'blblue'        : 104,
    'blmagenta'     : 105,
    'blcyan'        : 106,

    # Reset foreground and background colors
    'rcolor'        : 39,
    'rbackground'   : 49,
}
